"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const Logger_1 = require("../lib/Logger");
const Api_1 = require("./Api");
const Urls_1 = require("./Urls");
const fs = require("fs");
const crypto_1 = require("crypto");
const FormData = require('form-data');
class ZebAgent {
    _refreshToken;
    _header;
    _urls;
    _accessToken;
    _projectKey;
    _reportBaseUrl;
    _concurrentTasks;
    _enabled;
    _api;
    _defaultConcurrentTask = 10;
    _maximumConcurrentTask = 20;
    constructor(config) {
        this._accessToken = process.env.ZEB_API_KEY;
        this._projectKey = config.projectKey;
        this._reportBaseUrl = config.reporterBaseUrl;
        if (config.enabled) {
            this._enabled = true;
        }
        else {
            this._enabled = false;
        }
        this._concurrentTasks = config.concurrentTasks || this._defaultConcurrentTask;
        if (this._concurrentTasks > this._maximumConcurrentTask) {
            this._concurrentTasks = this._maximumConcurrentTask;
        }
        this._urls = new Urls_1.default(this._projectKey, this._reportBaseUrl);
        this._api = new Api_1.default(2, 1000);
    }
    async initialize() {
        const payload = {
            refreshToken: this._accessToken,
        };
        let endpoint = this._urls.urlRefresh();
        let r = await this._api.post({
            url: endpoint.url,
            payload: payload,
            expectedStatusCode: endpoint.status,
        });
        if (!r) {
            throw new Error('Failed to obtain refresh token');
        }
        this._refreshToken = `Bearer ${r.data.authToken}`;
        this._header = {
            headers: {
                Authorization: this._refreshToken,
            },
        };
        Logger_1.default.log(`initialize complete: obtained refreshToken ${this._refreshToken.substring(0, 10)}*****}`);
        Logger_1.default.log(`BASE_URL => ${this._reportBaseUrl}`);
        Logger_1.default.log(`ACCESS_TOKEN => ${this._accessToken.substring(0, 4)}*****`);
        Logger_1.default.log(`PROJECT_KEY => ${this._projectKey}`);
    }
    get concurrency() {
        return this._concurrentTasks;
    }
    get isEnabled() {
        return this._enabled;
    }
    get projectKey() {
        return this._projectKey;
    }
    get baseUrl() {
        return this._reportBaseUrl;
    }
    async startTestRun(payload) {
        let endpoint = this._urls.urlRegisterRun();
        let r = await this._api.post({
            url: endpoint.url,
            payload: payload,
            expectedStatusCode: endpoint.status,
            config: this._header,
        });
        return r;
    }
    async startTestExecution(testRunId, payload) {
        let endpoint = this._urls.urlStartTest(testRunId);
        let r = await this._api.post({
            url: endpoint.url,
            payload: payload,
            expectedStatusCode: endpoint.status,
            config: this._header,
        });
        return r;
    }
    async finishTestExecution(testRunId, testId, payload) {
        try {
            let endpoint = this._urls.urlFinishTest(testRunId, testId);
            let r = await this._api.put({
                url: endpoint.url,
                payload: payload,
                expectedStatusCode: endpoint.status,
                config: this._header,
            });
            return r;
        }
        catch (error) {
            console.log(error);
        }
    }
    async finishTestRun(testRunId, payload) {
        const endpoint = this._urls.urlFinishRun(testRunId);
        let r = await this._api.put({
            url: endpoint.url,
            payload: payload,
            expectedStatusCode: endpoint.status,
            config: this._header,
        });
        return r;
    }
    async attachScreenshot(testRunId, testId, screenshotsArray) {
        if (screenshotsArray.length === 0)
            return;
        const screenshotsPromises = screenshotsArray.map((screenshot) => {
            const file = fs.readFileSync(screenshot.path);
            const endpoint = this._urls.urlScreenshots(testRunId, testId);
            return this._api.post({
                url: endpoint.url,
                payload: Buffer.from(file),
                expectedStatusCode: endpoint.status,
                config: {
                    headers: {
                        Authorization: this._refreshToken,
                        'Content-Type': 'image/png',
                        'x-zbr-screenshot-captured-at': screenshot.timestamp,
                    },
                },
            });
        });
        const response = await Promise.all(screenshotsPromises);
        return response[0];
    }
    async attachTestArtifacts(testRunId, testId, artifactsAttachments) {
        if (artifactsAttachments.length === 0) {
            return;
        }
        try {
            const artifactsPromises = artifactsAttachments.map((file) => {
                const formData = new FormData();
                formData.append('file', fs.createReadStream(file.path));
                const endpoint = this._urls.urlTestArtifacts(testRunId, testId);
                const contentTypeHeader = formData.getHeaders()['content-type'];
                return this._api.post({
                    url: endpoint.url,
                    payload: formData,
                    expectedStatusCode: endpoint.status,
                    config: {
                        headers: {
                            Authorization: this._refreshToken,
                            'Content-Type': contentTypeHeader,
                            Accept: '*/*',
                        },
                    },
                });
            });
            const response = await Promise.all(artifactsPromises);
            return response[0];
        }
        catch (e) {
            console.log(e);
        }
    }
    async sendVideoArtifacts(testRunId, testSessionId, videoPathsArray) {
        if (videoPathsArray.length === 0) {
            return;
        }
        const promise = videoPathsArray.map((video) => {
            const formData = new FormData();
            formData.append('video', fs.createReadStream(video.path));
            const endpoint = this._urls.urlSessionArtifacts(testRunId, testSessionId);
            const contentTypeHeader = formData.getHeaders()['content-type'];
            const fileSize = this._getFileSizeInBytes(video.path);
            return this._api.post({
                url: endpoint.url,
                payload: formData,
                expectedStatusCode: endpoint.status,
                config: {
                    headers: {
                        Authorization: this._refreshToken,
                        'Content-Type': contentTypeHeader,
                        Accept: '*/*',
                        'x-zbr-video-content-length': fileSize,
                    },
                },
            });
        });
        const response = await Promise.all(promise);
        return response[0];
    }
    _getFileSizeInBytes = (filename) => {
        const stats = fs.statSync(filename);
        const fileSizeInBytes = stats.size;
        console.log('size', fileSizeInBytes);
        return fileSizeInBytes;
    };
    async addTestLogs(testRunId, logs) {
        if (logs.length <= 0)
            return;
        const endpoint = this._urls.urlSendLogs(testRunId);
        let r = await this._api.post({
            url: endpoint.url,
            payload: logs,
            expectedStatusCode: endpoint.status,
            config: this._header,
        });
        return r;
    }
    async addTestTags(testRunId, testId, items) {
        if (!items)
            return;
        let payload = {
            items,
        };
        const endpoint = this._urls.urlTestExecutionLabel(testRunId, testId);
        let r = await this._api.put({
            url: endpoint.url,
            payload: payload,
            expectedStatusCode: endpoint.status,
            config: this._header,
        });
        return r;
    }
    async addTestRunTags(testRunId, items) {
        if (!items)
            return;
        let payload = {
            items,
        };
        const endpoint = this._urls.urlTestRunLabel(testRunId);
        let r = await this._api.put({
            url: endpoint.url,
            payload: payload,
            expectedStatusCode: endpoint.status,
            config: this._header,
        }, 0);
        return r;
    }
    // this sends browser type to ZebRunner
    async startTestSession(options) {
        let payload = {
            sessionId: (0, crypto_1.randomUUID)(),
            initiatedAt: options.startedAt,
            startedAt: options.startedAt,
            desiredCapabilities: {
                browserName: options.browserCapabilities.browser.name,
                browserVersion: options.browserCapabilities.browser.version,
                platformName: options.browserCapabilities.os.name,
            },
            capabilities: {
                browserName: options.browserCapabilities.browser.name,
                browserVersion: options.browserCapabilities.browser.version,
                platformName: options.browserCapabilities.os.name,
            },
            testIds: [],
        };
        payload.testIds.push(options.testIds);
        try {
            const endpoint = this._urls.urlStartSession(options.testRunId);
            let r = await this._api.post({
                url: endpoint.url,
                payload: payload,
                expectedStatusCode: endpoint.status,
                config: this._header,
            });
            return r;
        }
        catch (error) {
            console.log(error);
        }
    }
    async finishTestSession(sessionId, testRunId, endedAt, testIds) {
        let payload = {
            endedAt: endedAt,
            testIds: [],
        };
        payload.testIds.push(testIds);
        const endpoint = this._urls.urlFinishSession(testRunId, sessionId);
        let r = await this._api.put({
            url: endpoint.url,
            payload: payload,
            expectedStatusCode: endpoint.status,
            config: this._header,
        });
        return r;
    }
}
exports.default = ZebAgent;
//# sourceMappingURL=ZebAgent.js.map